@@ -1,7 +1,7 @@
 use ruma::events::*;
 pub use ruma::{
     api::client::r0::sync::sync_events::*, identifiers::RoomName, presence::PresenceState,
-    serde::Raw, DeviceKeyAlgorithm, RoomAliasId, RoomId, UInt, UserId,
+    serde::Raw, DeviceKeyAlgorithm, EventId, RoomAliasId, RoomId, UInt, UserId,
 };
 use serde::{Deserialize, Serialize};
 use serde_json::{json, Value};
@@ -14,12 +14,13 @@ const BASE: &str = "/_matrix/client/v3/";
 #[derive(Clone, Debug, Serialize, Deserialize, Default)]
 pub struct StorageData {
     pub auth_token: String,
-    //pub room_to_aliases: BTreeMap<RoomId, Vec<RoomAliasId>>,
-    //pub alias_to_room: BTreeMap<RoomAliasId, RoomId>,
+    pub room_to_aliases: BTreeMap<RoomId, Vec<RoomAliasId>>,
+    pub alias_to_room: BTreeMap<RoomAliasId, RoomId>,
     pub joined_rooms: Vec<RoomId>,
     pub room_id_to_cananoical_alias: BTreeMap<RoomId, RoomAliasId>,
     pub room_to_name: BTreeMap<RoomId, Box<RoomName>>,
     pub room_to_direct_name: BTreeMap<RoomId, String>,
+    pub recent_event_ids: Vec<EventId>,
 }
 
 #[derive(Clone, Debug, Serialize, Deserialize, Default)]
@@ -450,7 +451,6 @@ impl Client {
             for event in joined_room.state.events.clone() {
                 //let ev = event.deserialize().unwrap();
                 if let Ok(ev) = event.deserialize() {
-                    /*
                     if let ruma::events::AnySyncStateEvent::RoomAliases(room_aliases_event) =
                         ev.clone()
                     {
@@ -462,7 +462,7 @@ impl Client {
                                     .insert(alias.clone(), room_id.clone())
                                     .unwrap();
                             }
-                            if !self.data.room_to_alesiases.contains_key(room_id) {
+                            if !self.data.room_to_aliases.contains_key(room_id) {
                                 self.data
                                     .room_to_aliases
                                     .insert(room_id.clone(), vec![alias])
@@ -477,10 +477,8 @@ impl Client {
                                         .unwrap();
                                 }
                             }
-                            //instructions.push(Instructions::SaveRoomAlias(room_id.clone(), alias));
                         }
                     }
-                    */
                     if let ruma::events::AnySyncStateEvent::RoomName(room_name_event) = ev.clone() {
                         if let Some(room_name) = room_name_event.content.name {
                             self.data.room_to_name.insert(room_id.clone(), room_name);
@@ -519,6 +517,16 @@ impl Client {
                 if let Ok(ruma::events::AnySyncRoomEvent::Message(message_event)) =
                     m.deserialize_as()
                 {
+                    let msg_event_id = message_event.event_id();
+                    if self.data.recent_event_ids.contains(msg_event_id) {
+                        return None;
+                    }
+                    if !self.data.recent_event_ids.contains(msg_event_id) {
+                        self.data.recent_event_ids.push(msg_event_id.clone());
+                    }
+                    if self.data.recent_event_ids.len() > 500 {
+                        self.data.recent_event_ids.remove(0);
+                    }
                     if let ruma::events::AnySyncMessageEvent::RoomMessage(room_message) =
                         message_event
                     {
